/* Convert Movie BYU format to PLG format */

/* Written by Bernie Roehl, April 1992 */

/* Reads from standard input, writes to standard output.  If there's a command
   line parameter, it's a scale factor (floating point).  If there's a second
   command line parameter, it's used as the name in the resulting .PLG file.

   Note that BYU files seem to include a "base" consisting of a single very
   large poly; this can be very disconcerting when it swings up at you and
   obscures your view.

   BYU files also seem to have only one surface/poly; the -d flag causes
   them to be "doubled" -- i.e. there'll be two polys, facing in opposite
   directions, for each input poly.
 */

#include <stdio.h>
#include <string.h>
#include <math.h>

float scale = 1.0;

int color = 0;
int doublesided = 0;  /* if non-zero, generate polys for both sides */

void main(int argc, char *argv[])
	{
	while (argc > 1)
		if (argv[1][0] == '-') {
			switch (argv[1][1]) {
				case 'd': doublesided = 1; break;
				default: fprintf(stderr, "Ignoring unknown option '%s'\n", argv[1]);
				}
			++argv;
			--argc;
			}
		else
			break;
	if (argc > 1) scale = atof(argv[1]);
	process(stdin, (argc > 2) ? argv[2] : "byuinput", stdout);
	}

#define MAXPOINTS 100  /* maximum number of points in a polygon */
int points[MAXPOINTS];

process(FILE *in, char *name, FILE *out)
	{
	int nparts, nverts, npolys, nedges;
	fscanf(in, "%d %d %d %d", &nparts, &nverts, &npolys, &nedges);
	fprintf(out, "%s %d %d\n", name, nverts, doublesided ? npolys*2 : npolys);
	while (nparts--) fscanf(in, "%*d %*d");  /* skip part information */
	while (nverts--) {
		float x, y, z;
		fscanf(in, "%f %f %f", &x, &y, &z);
		fprintf(out, "%ld %ld %ld\n", (long) floor(x*scale), (long) floor(y*scale), (long) floor(z*scale));
		}
	while (npolys--) {
		int i, npoints;
		for (npoints = 0; npoints < MAXPOINTS; ++npoints) {
			int n;
			if (fscanf(in, "%d", &n) < 0) {
				printf("Early EOF!\n");
				exit(2);
				}
			points[npoints] = abs(n);
			if (n < 0) break;
			}
		fprintf(out, "%d %d", (color++) % 15 + 1, ++npoints);
		for (i = 0; i < npoints; ++i) fprintf(out, " %d", points[npoints-i-1]-1);
		fprintf(out, "\n");
		if (doublesided) {
			fprintf(out, "%d %d", (color++) % 15 + 1, npoints);
			for (i = 0; i < npoints; ++i) fprintf(out, " %d", points[i]-1);
			fprintf(out, "\n");
			}
		}
	return 0;
	}
