/* Combine vertices in a PLG file */

/* Written by Bernie Roehl, April 1992 */

/* Reads a PLG file from standard input, and writes one to standard output,
   combining identical vertices into single copies.  Takes an optional
   command line parameter that specifies how close two vertices can be to
   be considered "identical".
 */

#include <stdio.h>
#include <alloc.h>
#include <math.h>
#include <string.h>

struct _vertex { float x, y, z; } *v;

int *map;

float dist = 1;

float sqr(float x)
	{
	return x * x;
	}

close_to(float x1, float y1, float z1, float x2, float y2, float z2)
	{
	return sqr(x1-x2)+sqr(y1-y2)+sqr(z1-z2) < dist;
	}

static int line = 0;

void strip_comment(char *buff)
	{
	while (*buff && *buff != '#' && *buff != '*') ++buff;
	*buff = '\0';
	}

readline(char *buff)  /* read a line, strip the comment if any */
	{
	do {
		if (gets(buff) == NULL) return 1;
		++line;
		strip_comment(buff);
		} while (*buff == '\0');
	return 0;
	}

void end_of_file(char *place)
	{
	fprintf(stderr, "Early end-of-file: %s on line %d\n", place, line);
	exit(1);
	}

void main(int argc, char *argv[])
	{
	char buff[100], input[1000];
	float x, y, z;
	int nv, np;
	int i, j, n;
	if (argc > 1)
		dist = sqr(atof(argv[1]));
	if (readline(input)) end_of_file("reading header");
	sscanf(input, "%s %d %d", buff, &nv, &np);
	if ((v = calloc(nv, sizeof(struct _vertex))) == NULL) {
		fprintf(stderr, "Could not allocate memory for %d vertices!\n", nv);
		exit(2);
		}
	if ((map = calloc(nv, sizeof(int))) == NULL) {
		fprintf(stderr, "Could not allocate memory for %d mappings!\n", nv);
		exit(3);
		}
	for (i = n = 0; i < nv; ++i) {
		if (readline(input)) end_of_file("reading vertices");
		sscanf(input, "%f %f %f", &x, &y, &z);
		for (j = 0; j < n; ++j)
			if (close_to(v[j].x, v[j].y, v[j].z, x, y, z)) {
				map[i] = j;
				break;
				}
		if (j >= n) {
			v[n].x = x;  v[n].y = y;  v[n].z = z;
			map[i] = n++;
			}
		}
	printf("%s %d %d\n", buff, n, np);
	for (i = 0; i < n; ++i)
		printf("%ld %ld %ld\n", (long) floor(v[i].x), (long) floor(v[i].y), (long) floor(v[i].z));
	for (i = 0; i < np; ++i) {
		int npts;
		unsigned color;
		char *p;
		if (readline(input)) end_of_file("reading polys");
		color = (unsigned) strtoul(input, &p, 0);
		npts = (int) strtol(p, &p, 0);
		printf("0x%04.4X %d", color, npts);
		for (j = 0; j < npts; ++j) {
			int pt;
			pt = (int) strtol(p, &p, 0);
			printf(" %d", map[pt]);
			}
		printf("\n");
		}
	}
