/* Convert a Sun "shaded" file into a PLG file */

/* Written by Bernie Roehl, July 1992 */

/* Reads from standard input, writes to standard output.  If there's a command
   line parameter, it's a scale factor (floating point).  If there's a second
   command line parameter, it's used as the name in the resulting .PLG file.

   If a -d is specified as the first parameter, two polys are output for
   each input poly (i.e. the input polys were intended to be double-sided).
 */

#include <stdio.h>
#include <string.h>
#include <alloc.h>
#include <math.h>

#define MAXVERTS 100  /* maximum number of vertices per polygon */
int verts[MAXVERTS];

int color = 5;

void main(int argc, char *argv[])
	{
	char buff[1000];
	int doublesided = 0;
	float scale = 1.0;
	char *name = "surf";
	int nv, np;
	while (argc > 1 && *argv[1] == '-') {
		if (argv[1][1] == 'd') doublesided = 1;
		++argv; --argc;
		}
	if (argc > 1) scale = atof(argv[1]);
	if (argc > 2) name = argv[2];
	printf("# Created by SHD2PLG\n");
	gets(buff);  /* read control line */
	sscanf(buff, "%d %d", &nv, &np);
	printf("%s %d %d\n", name, nv, doublesided ? (np * 2) : np);
	gets(buff);  /* skip limits */
	while (nv--) {
		float x, y, z;
		gets(buff);
		sscanf(buff, "%f %f %f", &x, &y, &z);
		printf("%ld %ld %ld\n", (long) floor(x * scale), (long) floor(y * scale), (long) floor(z * scale));
		}
	while (np--) {
		int npts, i, vnum;
		char *p;
		gets(buff);
		npts = strtol(buff, &p, 0);
		vnum = 0;
		while (npts-- && vnum < MAXVERTS)
			verts[vnum++] = ((int) strtol(p, &p, 0))-1;
		printf("0x%4.4X %d", 0x10A8 | ((color & 0x0F) << 8), vnum);
		for (i = 0; i < vnum; ++i) printf(" %d", verts[i]);
		printf("\n");
		if (doublesided) {
			printf("0x%4.4X %d", 0x10A8 | ((color & 0x0F) << 8), vnum);
			for (i = vnum-1; i >= 0; --i) printf(" %d", verts[i]);
			printf("\n");
			}
		}
	}

