/* Convert a SURFMODL file into a PLG file */

/* Written by Bernie Roehl, July 1992 */

/* Reads from standard input, writes to standard output.  If there's a command
   line parameter, it's a scale factor (floating point).  If there's a second
   command line parameter, it's used as the name in the resulting .PLG file.
 */

#include <stdio.h>
#include <string.h>
#include <alloc.h>
#include <math.h>

void readline(char *buff)
	{
	do {
		if (gets(buff) == NULL) break;
		} while(buff[0] == '*');
	}

void main(int argc, char *argv[])
	{
	char buff[1000];
	int *verts;  /* stores the vertices in a poly */
	float scale = 1.0;
	char *name = "surf";
	int nm, nv, np, maxv, nsides;
	if (argc > 1) scale = atof(argv[1]);
	if (argc > 2) name = argv[2];
	printf("# Created by SRF2PLG\n");
	readline(buff);  /* ignore title line */
	readline(buff);  /* ignore version number */
	readline(buff);  /* read control line */
	sscanf(buff, "%d %d %d %d %d", &nm, &nv, &np, &maxv, &nsides);
	if ((verts = calloc(sizeof(int), maxv)) == NULL) {
		fprintf(stderr, "Couldn't allocate memory\n");
		exit(1);
		}
	printf("%s %d %d\n", name, nv, np * nsides);
	while (nm--) readline(buff);  /* skip over materials */
	while (nv--) {
		float x, y, z;
		readline(buff);
		sscanf(buff, "%f %f %f", &x, &y, &z);
		printf("%ld %ld %ld\n", (long) floor(x * scale), (long) floor(y * scale), (long) floor(z * scale));
		}
	while (np--) {
		int color, npts, i, vnum;
		char *p;
		readline(buff);
		npts = strtol(buff, &p, 0);
		color = strtol(p, &p, 0);
		vnum = 0;
		while (npts--)
			verts[vnum++] = ((int) strtol(p, &p, 0))-1;
		printf("0x%4.4X %d", 0x10A8 | ((color & 0x0F) << 8), vnum);
		for (i = 0; i < vnum; ++i) printf(" %d", verts[i]);
		printf("\n");
		if (nsides == 2) {
			printf("0x%4.4X %d", 0x10A8 | ((color & 0x0F) << 8), vnum);
			for (i = vnum-1; i >= 0; --i) printf(" %d", verts[i]);
			printf("\n");
			}
		}
	free(verts);
	}

